using System;
using System.Windows.Forms;

namespace Dapfor.Net.Grid.Benchmark
{
    internal class LoadBalancingGrid : Ui.Grid
    {
        private bool isLimitFound;
        private TimeSpan lastPeriodBetweenTicks;
        private DateTime lastTick;
        private long limitCount;
        private int multiplier = 1;
        private long tickCount;
        private Timer timer = new Timer();


        public LoadBalancingGrid()
        {
            timer.Interval = 10;
            lastTick = DateTime.UtcNow;
            timer.Tick += timer_Tick;
        }

        public bool IsLimitFound
        {
            get { return isLimitFound; }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && timer != null)
            {
                timer.Dispose();
                timer = null;
            }

            base.Dispose(disposing);
        }

        public void StartLoadBalancingProcess()
        {
            tickCount = 0;
            limitCount = 0;
            multiplier = 1;
            isLimitFound = false;
            timer.Start();
        }

        public void StopLoadBalancingProcess()
        {
            tickCount = 0;
            limitCount = 0;
            multiplier = 1;
            isLimitFound = false;
            timer.Stop();
        }

        public event EventHandler ProcessUpdate;

        private void timer_Tick(object sender, EventArgs e)
        {

            for (int i = 0; i < multiplier; ++i)
            {
                if (ProcessUpdate != null)
                {
                    //If this method is called too often, than the distance between two timer ticks will incrase
                    ProcessUpdate(this, EventArgs.Empty);
                }
            }

            //Compute a period between two timer ticks - if it is too big - the Application works at the maximum of CPU resources
            lastPeriodBetweenTicks = DateTime.UtcNow - lastTick;
            lastTick = DateTime.UtcNow;

            //Do 10 empty cycles
            if (tickCount++ > 10)
            {
                //Balance the speed
                AdjustRefreshSpeed();
            }
        }

        private void AdjustRefreshSpeed()
        {
            if (lastPeriodBetweenTicks.TotalMilliseconds > 3*timer.Interval)
            {
                if (limitCount++ > 5)
                {
                    isLimitFound = true;
                }
            }

            //Start load balancing
            if (isLimitFound)
            {
                //The period between two timer ticks is too big - decrase the speed
                if (lastPeriodBetweenTicks.TotalMilliseconds > 2*timer.Interval)
                {
                    multiplier -= 1;
                }
                else
                {
                    //The period between two timer ticks is resonable - incrase the speed
                    multiplier += 1;
                }
                multiplier = Math.Max(multiplier, 0);
            }
            else
            {
                //Look for a limit
                multiplier += 3;
            }
        }
    }
}